; procedure to determine the with and the peak of a cloud PDF
;    from a noisy measurement
function noisecorrectpdf, feld, noiseamp, quality=quality, $
	directory=directory, plotminimum=plotminimum

; Input: 		feld = input array 
;			noiseamp = rms of the noise in the same units
; Optional input:	directory = directory where to read the precomputed
;			  files: noise-to-peak-ratio.fits, noisy-width.fits
; Output:		noisecorrectpdf = 2-element array for peak position
; 		           and standard deviation of log-normal PDF
; Optional output: 	quality = field of deviations from optimal fit
;			plotminimum = produce contour plot for match up 
;			   to given level

; sanity check
fsize=n_elements(feld)
if fsize lt 100 then $
	message, 'Input field too small to evaluate statistics'
if n_elements(directory) eq 0 then directory='.'

; definitions
binsize=0.1
bn=binsize/alog(10.0)

; Read lookup fields of precomputed values - assume same dimensions
etafield=readfits(directory+"/noise-to-peak-ratio.fits",hdr)
widthfield=readfits(directory+"/noisy-width.fits",hdr)
noiseaxis=headerscl(hdr,yscale=cloudwidth)
nnoise=n_elements(noiseaxis)
nwidth=n_elements(cloudwidth)

; Compute PDF
hist3=histogram(alog(feld),binsize=bn,location=nhist3,/nan)/binsize
nhist3=nhist3+0.5*bn
; make zeros numerical to allow for fit
ninv=where(hist3 gt 0,nvalid)
fhist3=double(hist3[ninv])/double(fsize)
nhist3=nhist3[ninv]

; fit Gaussian
testarr=fitpeak(nhist3,fhist3,downhilllimit=0.5)
center=testarr[1]
width=testarr[2]
; Compute noise amplitude relative to peak
relnoise=noiseamp/exp(center)

; compare with field of solutions
quality=sqrt((etafield-relnoise)^2+(widthfield-width)^2)
best=(mywhere(quality eq min(quality)))[0,*]
besteta=noiseaxis[best[0]]
bestwidth=cloudwidth[best[1]]

; control plot
if keyword_set(plotminimum) then $
  contour, (quality < plotminimum), noiseaxis, cloudwidth, /downhill, /follow, $
    xtitle="!4r!3!Dnoise!N/N!Dpeak!N",ytitle="!4r!3!Dcloud!N", $
    levels=[0.0625,0.125,0.25,0.5,1.0]*plotminimum, $
    title="Fit results for !4r!3!Dnoise!N=" $
	+strtrim(string(noiseamp,format="(G10.3)"),2)+": N!Dpeak!N=" $
	+strtrim(string(noiseamp/besteta,format="(G10.3)"),2)+", !4r!3!Dcloud!N=" $
	+strtrim(string(bestwidth,format="(G10.3)"),2)

; return minimum
return, [noiseamp/besteta,bestwidth]
end

